#!/bin/sh
#
# Main program entry point.
#
# Copyright 2023 Andrew Wood
#
# License GPLv3+: GNU GPL version 3 or later; see `docs/COPYING'.
#

# Adjust the default path.
export PATH="/sbin:/bin:/usr/sbin:/usr/bin:$PATH"

# Load the output and function override functions.
. "${COMPONENTDIR}/output.sh"
. "${COMPONENTDIR}/override.sh"

action=""
actionClass=""
optionNonInteractive=0
optionVerbose=0
optionQuiet=0
minArgs=0
maxArgs=0

# Default to non-interactive mode if stdin is not a terminal.
test -t 0 || optionNonInteractive=1

# Parse the command-line options to apply options and select the action.
while test -n "$1" && test -z "${action}"; do
	arg="$1"
	shift
	case "${arg}" in
	"-h"|"--help"|"help")		action="help" ;;
	"-V"|"--version"|"version")	action="version" ;;
	"-n"|"--non-interactive")	optionNonInteractive=1 ;;
	"-i"|"--interactive")		optionNonInteractive=0 ;;
	"-v"|"--verbose")		optionVerbose=$((1+optionVerbose)) ;;
	"-vv"|"--debug")		optionVerbose=$((2+optionVerbose)) ;;
	"-q"|"--quiet")			optionQuiet=$((1+optionQuiet)) ;;
	"-qq"|"--very-quiet")		optionQuiet=$((2+optionQuiet)) ;;
	"--"*)				actError "UNKNOWN-OPTION" "${arg}"; exit 1 ;;
	"-"[a-z]*)
		for optionLetter in $(echo "${arg#-}" | sed 's,\(.\),\1 ,g'); do
			case "${optionLetter}" in
			'h')	action="help" ;;
			'V')	action="version" ;;
			'n')	optionNonInteractive=1 ;;
			'i')	optionNonInteractive=0 ;;
			'v')	optionVerbose=$((1+optionVerbose)) ;;
			'q')	optionQuiet=$((1+optionQuiet)) ;;
			*)	actError "UNKNOWN-OPTION" "${optionLetter}"; exit 1 ;;
			esac
		done
		;;
	"-"*)				actError "UNKNOWN-OPTION" "${arg}"; exit 1 ;;
	"package-install")		minArgs=1; maxArgs=0; action="package-install" ;;
	"package-update")		minArgs=1; maxArgs=0; action="package-upgrade" ;;
	"package-upgrade")		minArgs=1; maxArgs=0; action="package-upgrade" ;;
	"package-downgrade")		minArgs=1; maxArgs=0; action="package-downgrade" ;;
	"package-remove")		minArgs=1; maxArgs=0; action="package-remove" ;;
	"package-reinstall")		minArgs=1; maxArgs=0; action="package-reinstall" ;;
	"package-info")			minArgs=1; maxArgs=0; action="package-info" ;;
	"package-files")		minArgs=1; maxArgs=0; action="package-files" ;;
	"package-owning-file")		minArgs=1; maxArgs=0; action="package-owning-file" ;;
	"package-cache-update")		minArgs=0; maxArgs=0; action="package-cache-update" ;;
	"package-cache-clean")		minArgs=0; maxArgs=0; action="package-cache-clean" ;;
	"package-update-list")		minArgs=0; maxArgs=0; action="package-upgrade-list" ;;
	"package-upgrade-list")		minArgs=0; maxArgs=0; action="package-upgrade-list" ;;
	"package-upgrade-all")		minArgs=0; maxArgs=0; action="package-upgrade-all" ;;
	"package-update-all")		minArgs=0; maxArgs=0; action="package-upgrade-all" ;;
	"package-list-installed")	minArgs=0; maxArgs=0; action="package-list-installed" ;;
	"service-start")		minArgs=1; maxArgs=1; action="service-start" ;;
	"service-stop")			minArgs=1; maxArgs=1; action="service-stop" ;;
	"service-restart")		minArgs=1; maxArgs=1; action="service-restart" ;;
	"service-try-restart")		minArgs=1; maxArgs=1; action="service-try-restart" ;;
	"service-reload")		minArgs=1; maxArgs=1; action="service-reload" ;;
	"service-enable")		minArgs=1; maxArgs=1; action="service-enable" ;;
	"service-disable")		minArgs=1; maxArgs=1; action="service-disable" ;;
	"service-mask")			minArgs=1; maxArgs=1; action="service-mask" ;;
	"service-unmask")		minArgs=1; maxArgs=1; action="service-unmask" ;;
	"service-is-known")		minArgs=1; maxArgs=1; action="service-is-known" ;;
	"service-is-running")		minArgs=1; maxArgs=1; action="service-is-running" ;;
	"service-is-enabled")		minArgs=1; maxArgs=1; action="service-is-enabled" ;;
	"service-is-masked")		minArgs=1; maxArgs=1; action="service-is-masked" ;;
	"service-list-all")		minArgs=0; maxArgs=0; action="service-list-all" ;;
	*)				actError "UNKNOWN-ACTION" "${arg}"; exit 1 ;;
	esac
done

# Early exit if no action was chosen.
test -z "${action}" && { actError "NO-ACTION"; exit 1; }

# Early exit if the number of parameters is incorrect.
test "${minArgs}" -gt 0 && test -z "$1" && { actError "MISSING-ARGUMENT" "${action}"; exit 1; }
test "${minArgs}" -eq 0 && test "${maxArgs}" -eq 0 && test -n "$1" && { actError "UNEXPECTED-ARGUMENT" "${action}"; exit 1; }

# Store the arguments as a string for hooks to view.
actionArguments="$*"

# Set the action class to the first word of the action.
actionClass="${action%%-*}"

# Camel-case the action and class, so "service-is-masked" becomes
# "actServiceIsMasked", to get the appropriate function name to call.
actionFunction="act"
for word in $(echo "${action}" | tr '-' ' '); do
	actionFunction="${actionFunction}$(echo "${word}" | cut -b 1 | tr '[:lower:]' '[:upper:]')${word#?}"
done
# The action class is just one word.
actionClassFunction="act$(echo "${actionClass}" | cut -b 1 | tr '[:lower:]' '[:upper:]')${actionClass#?}"

# Honour the "--quiet" and "--quiet --quiet" settings.
test "${optionQuiet}" -gt 0 && exec 2>/dev/null
test "${optionQuiet}" -gt 1 && exec >/dev/null

actDebug "actionClass=[${actionClass}], actionClassFunction=[${actionClassFunction}]"
actDebug "action=[${action}], actionFunction=[${actionFunction}]"
actDebug "actionArguments=[$*]"
actDebug "optionNonInteractive=${optionNonInteractive}"
actDebug "optionVerbose=${optionVerbose}"
actDebug "optionQuiet=${optionQuiet}"

# Run the pre-action hooks.
. "${COMPONENTDIR}/hooks.sh"
actHooks "pre" "all" "$@"
actHooks "pre" "${actionClass}" "$@"
test "${actionClass}" = "${action}" || actHooks "pre" "${action}" "$@"

exitStatus=0

# Run the class action.
if test -n "${actionClass}" && test -e "${COMPONENTDIR}/${actionClass}.sh"; then
	actDebug "loading action class script: ${COMPONENTDIR}/${actionClass}.sh"
	. "${COMPONENTDIR}/${actionClass}.sh"
fi
if type "${actionClassFunction}" >/dev/null 2>&1; then
	actDebug "running action class function: ${actionClassFunction}"
	"${actionClassFunction}" "$@"
	returnCode=$?
	test "${returnCode}" -gt 0 && test "${exitStatus}" -eq 0 && exitStatus="${returnCode}"
	actDebug "exitStatus=${exitStatus}"
fi

# Run the specific action.
if ! test "${action}" = "${actionClass}"; then
	if test -e "${COMPONENTDIR}/${action}.sh"; then
		actDebug "loading action script: ${COMPONENTDIR}/${action}.sh"
		. "${COMPONENTDIR}/${action}.sh"
	fi
	if type "${actionFunction}" >/dev/null 2>&1; then
		actDebug "running action function: ${actionFunction}"
		"${actionFunction}" "$@"
		returnCode=$?
		test "${returnCode}" -gt 0 && test "${exitStatus}" -eq 0 && exitStatus="${returnCode}"
		actDebug "exitStatus=${exitStatus}"
	else
		actError "UNSUPPORTED-ACTION" "${action}"
		exitStatus=2
	fi
fi

# Run the post-action hooks.
test "${actionClass}" = "${action}" || actHooks "post" "${action}" "$@"
actHooks "post" "${actionClass}" "$@"
actHooks "post" "all" "$@"

exit ${exitStatus}
